'use strict';

const asyncp = require('paradox-asyncp');
const Promise = require('bluebird');

class EventEmitter {
    constructor() {
        this.events = {};
    }

    on(event, callback, priority) {

        if(!(event in this.events)) {
            this.events[event] = [];
        }

        this.events[event].push({
            callback,
            priority: priority || 0,
        });

        return this;
    }

    emit(event, params) {
        let events = this.events[event] || [];

        if(events.length < 1) {
            return Promise.resolve(undefined);
        }

        let lastResp;

        events.sort((a, b) => {
            return a.priority == b.priority ? 0 : (a.priority > b.priority ? 1 : -1);
        });

        return asyncp
            .eachLimit(events, 1, event => {
                if(typeof lastResp != 'undefined') {
                    return Promise.resolve(undefined);
                }

                return Promise
                    .try(() => {
                        return event.callback.apply(this, params);
                    })
                    .then(res => {
                        lastResp = res;
                    });
            })
            .then(() => lastResp);
    }
}

module.exports = EventEmitter;